/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly redrawing to the Java graphics window.

   This program sleeps for a short time between animation frames.
*/
public class Animation
{
   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("assets/apple.obj"));
      Model m2 = new ObjSimpleModel(new File("assets/cow.obj"));
      Model m3 = new ObjSimpleModel(new File("assets/cessna.obj"));
      Model m4 = new ObjSimpleModel(new File("assets/galleon.obj"));
      Model m5 = new ObjSimpleModel(new File("assets/teapot.obj"));
      Model m6 = new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj"));
      Model m7 = new ObjSimpleModel(new File("assets/great_rhombicosidodecahedron.obj"));


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (Model m : scene.modelList)
         for (LineSegment ls : m.lineSegmentList)
         {
            ls.v[0].z -= 3;
            ls.v[1].z -= 3;
         }


      // Create a InteractiveFrame (with a FrameBuffer) to render our scene into.
      int width  = 512;
      int height = 512;
      InteractiveFrame app = new InteractiveFrame("Renderer 1", width, height);

      for (int i = 0; i <= 500; i++)
      {
         // Render again.
         app.getFrameBuffer().clearFB(Color.darkGray);
         Pipeline.render(scene, app.getFrameBuffer());
         app.repaint();
         try{Thread.sleep(30);}catch(InterruptedException e){return;}

         // update the scene
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            ls.v[0].x += 0.3;  // move right
            ls.v[1].x += 0.3;
            ls.v[0].y += 0.1;  // move up
            ls.v[1].y += 0.1;
            ls.v[0].z -= 0.1;  // move back
            ls.v[1].z -= 0.1;
         }
         for (LineSegment ls : scene.modelList.get(1).lineSegmentList)
         {
            ls.v[0].x -= 0.3;  // move left
            ls.v[1].x -= 0.3;
            ls.v[0].y -= 0.1;  // move down
            ls.v[1].y -= 0.1;
            ls.v[0].z -= 0.1;  // move back
            ls.v[1].z -= 0.1;
         }
         for (LineSegment ls : scene.modelList.get(2).lineSegmentList)
         {
            ls.v[0].x += 0.01; // move right
            ls.v[1].x += 0.01;
         }
      }
   }
}
