/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**


*/
public class InteractiveHorizonDemo
{
   private static int n = 9;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, keys to move the horizon point along the x and y axes.");
      System.out.println("Use the z/Z, keys to change the length of the lines.");
      System.out.println("Use the n/N, keys to change the number of lines.");

      // Create the Model object.
      final Model model = new PrivateModel(n);


      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 1", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
            }
            else if ('p' == c)
            {
               scene.camera.perspective = ! scene.camera.perspective;
            }
            else if ('x' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].x += 0.1;
               }
            }
            else if ('X' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].x -= 0.1;
               }
            }
            else if ('y' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].y += 0.1;
               }
            }
            else if ('Y' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].y -= 0.1;
               }
            }
            else if ('z' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].z += 1.0;
               }
            }
            else if ('Z' == c)
            {
               for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
               {
                  ls.v[1].z -= 1.0;
               }
            }
            else if (('n' == c) || ('N' == c))
            {
               if ('n' == c)
                  n--;
               else
                  n++;
               scene.modelList.set(0,  new PrivateModel(n));
            }
            // Render again.
            this.getFrameBuffer().clearFB(Color.darkGray);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.darkGray);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}


class PrivateModel extends Model
{
   PrivateModel(int n)
   {
      super();

      // Add geometry to the Model.
      Vertex[][][] v = new Vertex[n][n][2];

      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < n; j++)
         {
            v[i][j][0] = new Vertex( -1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -1 );
            v[i][j][1] = new Vertex( -1.0 + (2.0*i)/(n-1), -1.0 + (2.0*j)/(n-1), -10 );
            this.addLineSegment( new LineSegment(v[i][j][0], v[i][j][1]) );
         }
      }
   }
}
