/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
                   y
                   |
                   |
                   | v[4]
                 1 +-----------------+ v[5]=(1,1,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v[1]               |
           |  v[0] +---------|-------+------> x           |
           |      /          |      /1                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         1 +-----------------+                         /
         / v[3]=(0,0,1)      v[2]=(1,0,1)            z
       /
     /
    z
*/
public class OfflineCube
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      Model model = new Model();

      // Add geometry to the Model.
      Vertex[] v = new Vertex[8];

      v[0] = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      v[1] = new Vertex(1.0, 0.0, 0.0);
      v[2] = new Vertex(1.0, 0.0, 1.0);
      v[3] = new Vertex(0.0, 0.0, 1.0);
      v[4] = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      v[5] = new Vertex(1.0, 1.0, 0.0);
      v[6] = new Vertex(1.0, 1.0, 1.0);
      v[7] = new Vertex(0.0, 1.0, 1.0);

      model.addLineSegment( new LineSegment(v[0], v[1]) );  // bottom face
      model.addLineSegment( new LineSegment(v[1], v[2]) );  // bottom face
      model.addLineSegment( new LineSegment(v[2], v[3]) );  // bottom face
      model.addLineSegment( new LineSegment(v[3], v[0]) );  // bottom face
      model.addLineSegment( new LineSegment(v[4], v[5]) );  // top face
      model.addLineSegment( new LineSegment(v[5], v[6]) );  // top face
      model.addLineSegment( new LineSegment(v[6], v[7]) );  // top face
      model.addLineSegment( new LineSegment(v[7], v[4]) );  // top face
      model.addLineSegment( new LineSegment(v[0], v[4]) );  // back face
      model.addLineSegment( new LineSegment(v[1], v[5]) );  // back face
      model.addLineSegment( new LineSegment(v[3], v[7]) );  // front face
      model.addLineSegment( new LineSegment(v[2], v[6]) );  // front face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Push the model away from where the camera is.
      for (int i = 0; i < v.length; i++)
      {
         v[i].z -= 2;
      }

      // log interesting information to standard output
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      for (int i = 0; i <= 100; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCube_Frame%03d.ppm", i));

         // update the scene by translating in
         // camera space each of the 8 vertices
         for (int j = 0; j < v.length; j++)
         {
            v[j].x += 0.1;
          //v[j].z -= 0.01;
         }
      }
   }
}