/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
                   y
                   |
                   |
                   | v[4]
                 1 +-----------------+ v[5]=(1,1,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v[1]               |
           |  v[0] +---------|-------+------> x           |
           |      /          |      /1                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         1 +-----------------+                         /
         / v[3]=(0,0,1)      v[2]=(1,0,1)            z
       /
     /
    z
*/
public class OfflineCubes
{
   public static void main(String[] args)
   {
      // Create Model objects to hold the geometry.
      Model model1 = new Model();
      Model model2 = new Model();

      // Add geometry to the Models
      // First cube
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      v[1] = new Vertex(1.0, 0.0, 0.0);
      v[2] = new Vertex(1.0, 0.0, 1.0);
      v[3] = new Vertex(0.0, 0.0, 1.0);
      v[4] = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      v[5] = new Vertex(1.0, 1.0, 0.0);
      v[6] = new Vertex(1.0, 1.0, 1.0);
      v[7] = new Vertex(0.0, 1.0, 1.0);

      // Second cube.
      Vertex[] w = new Vertex[8];
      w[0] = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      w[1] = new Vertex(2.0, 0.0, 0.0);
      w[2] = new Vertex(2.0, 0.0, 2.0);
      w[3] = new Vertex(0.0, 0.0, 2.0);
      w[4] = new Vertex(0.0, 2.0, 0.0);  // four vertices around the top face
      w[5] = new Vertex(2.0, 2.0, 0.0);
      w[6] = new Vertex(2.0, 2.0, 2.0);
      w[7] = new Vertex(0.0, 2.0, 2.0);

      // First cube.
      model1.addLineSegment( new LineSegment(v[0], v[1]) );  // bottom face
      model1.addLineSegment( new LineSegment(v[1], v[2]) );  // bottom face
      model1.addLineSegment( new LineSegment(v[2], v[3]) );  // bottom face
      model1.addLineSegment( new LineSegment(v[3], v[0]) );  // bottom face
      model1.addLineSegment( new LineSegment(v[4], v[5]) );  // top face
      model1.addLineSegment( new LineSegment(v[5], v[6]) );  // top face
      model1.addLineSegment( new LineSegment(v[6], v[7]) );  // top face
      model1.addLineSegment( new LineSegment(v[7], v[4]) );  // top face
      model1.addLineSegment( new LineSegment(v[0], v[4]) );  // back face
      model1.addLineSegment( new LineSegment(v[1], v[5]) );  // back face
      model1.addLineSegment( new LineSegment(v[3], v[7]) );  // front face
      model1.addLineSegment( new LineSegment(v[2], v[6]) );  // front face

      // Second cube.
      model2.addLineSegment( new LineSegment(w[0], w[1]) );  // bottom face
      model2.addLineSegment( new LineSegment(w[1], w[2]) );  // bottom face
      model2.addLineSegment( new LineSegment(w[2], w[3]) );  // bottom face
      model2.addLineSegment( new LineSegment(w[3], w[0]) );  // bottom face
      model2.addLineSegment( new LineSegment(w[4], w[5]) );  // top face
      model2.addLineSegment( new LineSegment(w[5], w[6]) );  // top face
      model2.addLineSegment( new LineSegment(w[6], w[7]) );  // top face
      model2.addLineSegment( new LineSegment(w[7], w[4]) );  // top face
      model2.addLineSegment( new LineSegment(w[0], w[4]) );  // back face
      model2.addLineSegment( new LineSegment(w[1], w[5]) );  // back face
      model2.addLineSegment( new LineSegment(w[3], w[7]) );  // front face
      model2.addLineSegment( new LineSegment(w[2], w[6]) );  // front face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Models to the Scene.
      scene.addModel(model1, model2);

      // Push the models away from where the camera is.
      for (int i = 0; i < v.length; i++)
      {
         v[i].z -= 4;
         w[i].z -= 4;
      }

      // log interesting information to standard output
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      for (int i = 0; i <= 100; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCubes_Frame%03d.ppm", i));

         // update the scene by translating in
         // camera space each of the vertices
         for (int j = 0; j < v.length; j++)
         {
            v[j].x -= 0.1;
            v[j].y -= 0.03;
          //v[j].z += 0.02;

            w[j].x += 0.1;
          //w[j].y += 0.4;
          //w[j].z -= 0.01;
         }
      }
   }
}