/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
                   y
                   |
                   |
                   | v[1]
                 1 +
                   |
                   |
                   |
                   |
                   |
                   |
                   |
                   |                  v[0]
              v[2] +-----------------+------> x
                  /                  1
                /
              /
            /
         1 +
         / v[2]
       /
     /
    z

   Render a single wireframe triangle. This is just about
   the simplest possible model. It is useful for debugging.
*/
public class OfflineTriangle
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // Add geometry to the Model.
      final Vertex[] v = new Vertex[3];

      v[0] = new Vertex(1.0, 0.0, 0.0);
      v[1] = new Vertex(0.0, 1.0, 0.0);
      v[2] = new Vertex(0.0, 0.0, 0.0);
    //v[2] = new Vertex(0.0, 0.0, 1.0);

      model.addLineSegment( new LineSegment(v[1], v[2]) );
      model.addLineSegment( new LineSegment(v[2], v[0]) );
      model.addLineSegment( new LineSegment(v[0], v[1]) );

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Push the model away from where the camera is.
      for (int i = 0; i < v.length; i++)
      {
         v[i].z -= 1;
      }

      // log interesting information to standard output
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Render our scene into the frame buffer.
      fb.clearFB(Color.darkGray);
      Pipeline.debug  = true;
      Rasterize.debug = true;
      Pipeline.render(scene, fb);
      fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", 0));
      Pipeline.debug  = false;
      Rasterize.debug = false;

      for (int i = 1; i <= 60; i++)
      {
         // update the scene by translating in
         // camera space each of the 8 vertices
         for (int j = 0; j < v.length; j++)
         {
            v[j].x += 0.2;
          //v[j].y -= 0.1;
            v[j].z -= 0.05;
         }
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", i));
      }
   }
}