/*

*/
package scene;

/**
    A Vertex object has three doubles which represent the coordinates
    of a point in 3-dimensional space.
<p>
    When a Vertex object is created in a client program, before the
    Vertex object moves down the graphics rendering pipeline, the
    coordinates in the Vertex will be in "camera coordinates".
<p>
    As a Vertex object moves down the graphics rendering pipeline,
    the coordinates in the Vertex will be transformed from one
    coordinate system to another.
*/
public class Vertex
{
   public double x, y, z;

   /**
      Construct a default vertex.
   */
   public Vertex()
   {
      set(0.0, 0.0, 0.0);
   }


   /**
      Construct a new vertex using the given x, y, and z coordinates.

      @param x x-coordinate of the new Vertex
      @param y y-coordinate of the new Vertex
      @param z z-coordinate of the new Vertex
   */
   public Vertex(double x, double y, double z)
   {
      set(x, y, z);
   }


   /**
      Construct a new vertex that is a copy of another vertex.

      @param v Vertex to make a copy of
   */
   public Vertex(Vertex v)
   {
      set(v.x, v.y, v.z);
   }


   /**
      Set the coordinates of this vertex.

      @param x new x-coordinate for this Vertex
      @param y new y-coordinate for this Vertex
      @param z new z-coordinate for this Vertex
   */
   public void set(double x, double y, double z)
   {
      this.x = x;
      this.y = y;
      this.z = z;
   }


   /**
      For debugging.

      @return String representation of this Vertex object
   */
   public String toString()
   {
      // Here is one way to get programmable precision and width.
      int p = 5;     // the precision for the following format string
      int t = p + 4; // the width for the following format string
      String format =      "(x,y,z) = (% "+t+"."+p+"f  % "+t+"."+p+"f  % "+t+"."+p+"f)\n";
      return String.format( format, x, y, z);
    //return String.format("(x,y,z) = (% .5f  % .5f  % .5f)", x, y, z);
   }
}
